<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xpath-default-namespace="http://www.destatis.de/schema/datml-raw/2.0/de"
    xmlns="http://www.destatis.de/schema/datml-raw/2.0/de"
    xmlns:func="http://www.destatis.de/schema/datml-raw/2.0/de/functions"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    exclude-result-prefixes="xs"
    version="2.0"
    >
    
    <!-- 
     **********************************************************************
        
     Stylesheet : res2res-gewerbe
     Modul      : res2res-gewerbe
     Version    : 0.6
     Variante   : -
     Typ        : XSLT (.xsl)
     Stand      : 2016-06-15
     Autor      : Michael Schaefer, Statistisches Bundesamt
     
     Parameter:
     ==========
     
     datml-raw-doc
                  URI des geprueften DatML/RAW Dokuments
     config-file
                  URI der Konfigurationsdatei fuer dieses Stylesheet
     debug
                  true|false; Default: false; Ausgabe von Debug-Meldungen
                
     Beschreibung:
     =============
     
     Ein DatML/RES Dokument wird in fuenf Phasen mittels Templates mit einem
     der folgenden Modi transformiert:
     
     - 'flag'
       Warnungen werden entsprechend der Konfiguration zu Fehlern propagiert.
     - 'message'
       Die Stati der Meldungen werden angepasst
     - 'document'
       Die Stati des Dokuments und der Nachrichten werden angepasst, und es
       werden UUIDs gesetzt.
     - 'data'
       Der uebergeordnete Pruefstatus fuer Daten wird gesetzt. 
     - 'text'
       Textersetzungen werden durchgefuehrt.
       
     Die Verarbeitung findet auch statt, wenn das zugehoerige DatML/RAW-
     Dokument nicht gelesen oder verarbeitet werden kann. In diesem Fall
     werden keine UUIDs gesetzt, da diese aus der Meldung gelesen werden.
     
     Version 0.3
       - Berichtseinheit-Id wird in Meldung ausgegeben

     Version 0.4
       - nachfolgende Anpassungen wurden Aufgrund der moeglichen uebergabe von 
         Streams vorgenommen: 
       		- Pruefung auf Dateiexistenz deaktiviert
       		- Variablen 'datml-raw-doc-intern' und 'config-file-intern' aufgenommen

     Version 0.5
       - Es wird eine zusaetzlich Phase 'data' ausgefuehrt. In dieser wird der
	     uebergeordnete Pruefstatus fuer Daten  entsprechend dem Pruefstatus des
	     Dokuments gesetzt.
	 
     Version 0.6
       - Das Propagieren von Warnungen kann fuer Berichtszeitraeume konfiguriert
         werden.
       - Parameter als Schalter fuer Debug-Meldungen
	 
     **********************************************************************
     -->
    
    <xsl:strip-space elements="*"/>
    <!-- Sollte ggf. per JAXP gesetzt werden -->
    <xsl:output indent="yes"/>
    
    <!-- Parameter -->
    
    <xsl:param name="datml-raw-doc" required="yes"/>
    <xsl:param name="config-file" required="yes"/>
    <xsl:param name="debug" required="no" select="false()" as="xs:boolean"/>
    
    <!-- Sekundaere Dokumente -->
    
    <xsl:variable name="datml-raw-doc-intern" select="$datml-raw-doc" as="document-node()"/>
    <xsl:variable name="config-file-intern" select="$config-file" as="document-node()"/>
    
    <!-- Zugriff auf Dokumente pruefen. Fuer den Aufruf aus Java (uebergabe von Streams) ist die Pruefung deaktiviert
        <xsl:variable name="raw-exists" as="xs:boolean" select="func:checkDocument($datml-raw-doc,'DatML/RAW Meldung')"/>
        <xsl:variable name="cfg-exists" as="xs:boolean" select="func:checkDocument($config-file,'Konfigurationsdatei')"/>
    -->
    
    <xsl:variable name="raw-exists" as="xs:boolean" select="true()"/>
    <xsl:variable name="cfg-exists" as="xs:boolean" select="true()"/>
    
    <!-- Auswertbarkeit des DatML/RAW-Dokumentes pruefen (es muss syntaktisch korrekt sein) -->
    
    <xsl:variable name="document-processible" as="xs:boolean" select="boolean(/DatML-RES-D/pruefprotokoll/pruefung/syntax[@pruefstatus eq 'OK'])"/>
    
    <!-- Ermitteln, ob Test- oder Produktivdaten vorliegen -->
    
    <xsl:variable name="processingMode" select="if (/DatML-RES-D/optionen/test) then 'test' else 'produktion'"/>
    
    <!-- Berichtszeitraum ermitteln -->
    
    <xsl:variable name="berichtszeitraum" select="/DatML-RES-D/pruefprotokoll/dokument/nachricht/meldung/berichtszeitraum" as="element()"/>
    
    <!-- Auswahl aller fuer den Berichtszeitraum moeglichen Schluessel-Umkodierungen -->
    
    <xsl:variable name="applicableCodeTransforms" as="element()*" select="
        for $t in (func:findAllCodeTransformsForPeriod(
            func:berichtszeitraum2String($berichtszeitraum),
            $config-file-intern/res2res-config/umkodierungen/kode
            ))
        return $t"
        />
    
    <!-- Verarbeitung des DatML/RES Protokolls -->
    
    <xsl:template match="/">
        
        <!-- Initiale Debug-Meldungen -->
        
        <xsl:if test="$debug">
            <xsl:message select="('Beginn der Verarbeitung',
                ', BZR=',func:berichtszeitraum2String($berichtszeitraum),
                ', MODUS=', $processingMode
                )"/>
            <xsl:message select="('Anwendbare Schluessel-Transformationen (',count($applicableCodeTransforms), '):')"/>
            <xsl:for-each select="$applicableCodeTransforms">
                <xsl:message select="(string(@alt), ' zu ',string(@neu), ', Text (optional): ',string(@text))"/>
            </xsl:for-each>
        </xsl:if>
        
        <!-- Abbruch wenn Konfigurationdatei nicht im Zugriff ist -->
        <!-- Fuer den Aufruf aus Java (uebergabe von Streams) deaktiviert
        <xsl:if test="not($cfg-exists)">
            <xsl:message terminate="yes" select="('Die Konfigurationsdatei',$config-file,'existiert nicht oder kann nicht verarbeitet werden.')"/>
        </xsl:if>
         -->
        
        <!-- Warnung wenn DatML/RAW-Dokument nicht im Zugriff ist -->
        <!-- Fuer den Aufruf aus Java (uebergabe von Streams) deaktiviert
        <xsl:if test="not($raw-exists)">
            <xsl:message select="('DatML-RAW Meldung ',$datml-raw-doc,'existiert nicht oder kann nicht verarbeitet werden.')"/>
        </xsl:if>
         -->
        
        <!-- Warnung wenn DatML/RAW-Dokument nicht im verarbeitbar ist -->
        
        <xsl:if test="not($document-processible)">
		    <xsl:choose>
                <xsl:when test="$debug">
                    <xsl:message select="('Die XGewerbe-Meldung ',$datml-raw-doc,' ist nicht valide; es werden nur textuelle Anpassungen des Protokolls durchgefuehrt.')"/>
                </xsl:when>
			    <xsl:otherwise>
                    <xsl:message select="('Die XGewerbe-Meldung ist nicht valide; es werden nur textuelle Anpassungen des Protokolls durchgefuehrt.')"/>
			    </xsl:otherwise>
			</xsl:choose>
        </xsl:if>
        
        <!-- Phase 'flag': Schluessel umkodieren -->
        
        <xsl:variable name="with-flags-propagated">
            <xsl:if test="$debug">
                <xsl:message select="'Start Phase ''flag'''"></xsl:message>
            </xsl:if>
            <xsl:apply-templates mode="flag"/>
        </xsl:variable>
        
        <!-- Phase 'message': Status der Meldung angepassen -->
        
        <xsl:variable name="with-messages-propagated">
            <xsl:if test="$debug">
                <xsl:message select="'Start Phase ''message'''"></xsl:message>
            </xsl:if>
            <xsl:apply-templates select="$with-flags-propagated" mode="message"/>
        </xsl:variable>
        
        <!-- Phase 'document': Status des Dokuments angepassen -->
        
        <xsl:variable name="with-document-propagated">
            <xsl:if test="$debug">
                <xsl:message select="'Start Phase ''document'''"></xsl:message>
            </xsl:if>
            <xsl:apply-templates select="$with-messages-propagated" mode="document"/>
        </xsl:variable>
        
        <!-- Phase 'data': Pruefstatus fuer Daten setzen -->
        
        <xsl:variable name="with-data-propagated">
            <xsl:if test="$debug">
                <xsl:message select="'Start Phase ''data'''"></xsl:message>
            </xsl:if>
            <xsl:apply-templates select="$with-document-propagated" mode="data"/>
        </xsl:variable>
        
        <!-- Phase 'text': Textersetzungen durchfuehren -->
        
        <xsl:if test="$debug">
            <xsl:message select="'Start Phase ''text'''"></xsl:message>
        </xsl:if>
        <xsl:apply-templates select="$with-data-propagated" mode="text"/>
        
    </xsl:template>
    
    <!-- ============ -->
    <!-- Phase 'data' -->
    <!-- ============ -->
    
    <!-- Uebergeordneten Pruefstatus fuer Daten setzen  -->
    
    <xsl:template match="pruefprotokoll/pruefung/daten" mode="data">
        <xsl:element name="{local-name()}">
            <xsl:variable name="subordinateStati" select="distinct-values(../../dokument/nachricht/meldung/pruefung/daten/@pruefstatus)" as="xs:string*"/>
            <xsl:variable name="mustPropagate" select="some $s in $subordinateStati satisfies $s eq 'fehlerhaft'" as="xs:boolean"/>
            <xsl:variable name="status" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus" as="xs:string"/>
            <xsl:if test="$debug">
                <xsl:message select="('Pruefstati Daten: ',$subordinateStati, ', ', if ($mustPropagate) then 'Pruefstatus wird auf fehlerhaft gesetzt!' else 'Keine Aenderung')"/>
            </xsl:if>
            <xsl:attribute name="pruefstatus" select="$status"/>
            <xsl:apply-templates mode="daten" select="* except (@pruefstatus)"/>
        </xsl:element>
    </xsl:template>
    
    <!-- ================ -->
    <!-- Phase 'document' -->
    <!-- ================ -->
    
    <!-- Dokumentstatus und Pruefstatus setzen -->
    
    <xsl:template match="dokument" mode="document">
        <xsl:variable name="subordinateStati" select="distinct-values(nachricht/meldung/@meldungsstatus)" as="xs:string*"/>
        <xsl:variable name="mustPropagate" select="every $s in $subordinateStati satisfies $s eq 'abgewiesen'" as="xs:boolean"/>
        <xsl:if test="$debug">
            <xsl:message select="('Meldungsstati: ',$subordinateStati, ', ', if ($mustPropagate) then 'Dokumentstatus wird auf abgewiesen gesetzt!' else 'Keine Aenderung')"/>
        </xsl:if>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="dokumentstatus" select="if ($mustPropagate) then 'abgewiesen' else @dokumentstatus"/>
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="document" select="* except (@dokumentstatus,@pruefstatus)"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Pruefstatus setzen Nachricht setzen -->
    
    <xsl:template match="nachricht" mode="document">
        <xsl:variable name="subordinateStati" select="distinct-values(meldung/@meldungsstatus)"/>
        <xsl:variable name="mustPropagate" select="every $s in $subordinateStati satisfies $s eq 'abgewiesen'" as="xs:boolean"/>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="document" select="* except @pruefstatus"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Attribut 'klasse' in 'absender/kennung' auf den Wert von hmm[@name='BerEmpfaenger'] setzen -->
    
    <xsl:template match="absender/kennung/@klasse" mode="document">
        <xsl:attribute name="klasse">
            <xsl:variable name="absender-klasse" select="
                if ($document-processible)
                then distinct-values(($datml-raw-doc-intern/DatML-RAW-D/nachricht//hmm[@name='BerEmpfaenger']/wert/text()))
                else ()
                "/>
            <xsl:choose>
                <xsl:when test="count($absender-klasse) gt 1">
                    <xsl:message terminate="yes">
                        ***** Dem Hilfsmerkmal [BerEmpfaenger] wurden unterschiedliche Werte zugeordnet: [<xsl:value-of select="string-join($absender-klasse,',')"/>] *****
                    </xsl:message>
                </xsl:when>
                <xsl:when test="empty($absender-klasse[1])">
                    <xsl:text>UNBEKANNT</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$absender-klasse[1]"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>
    
    <!-- =============== -->
    <!-- Phase 'message' -->
    <!-- =============== -->
    
    <!-- Meldungsebene modifizieren
        Wenn sich mindestens ein Fehlerschluessel auf Meldungsebene in der Menge der durch Umkodierung erzeugten
        Fehlerschluessel befindet, wird die Meldung als auf Datenebene fehlerhaft gekennzeichnet. Die Attribute
        meldungsstatus und pruefstatus werden entsprechend modifiziert.
    -->
    
    <xsl:template match="meldung" mode="message">
        <xsl:variable name="codes-from-flags" select="(distinct-values(fehler/schluessel/text()))" as="xs:string*"/>
        <xsl:variable name="mustPropagate" as="xs:boolean" select="some $w in $applicableCodeTransforms satisfies $w/@neu = $codes-from-flags"/>
        <xsl:if test="$debug">
            <xsl:message select="('Fehlerkodes: ',$codes-from-flags, ', ', if ($mustPropagate) then 'Meldungsstatus wird angepasst!' else 'Keine Aenderung')"/>
        </xsl:if>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="meldungsstatus" select="if ($mustPropagate) then 'abgewiesen' else @meldungsstatus"/>
            <xsl:attribute name="berichtseinheitID" select="@berichtseinheitID"/>
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="message" select="* except (@meldungsstatus,@pruefstatus)">
                <xsl:with-param name="mustPropagate" select="$mustPropagate" as="xs:boolean" tunnel="yes"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    
    <!-- Status der Datenpruefung setzen
        Wurden Warnungen zu Fehlern hochgewichtet, (s.o.), wird der Pruefstatus auf Datenebene modifiziert
    -->
    
    <xsl:template match="meldung/pruefung/daten/@pruefstatus" mode="message">
        <xsl:param name="mustPropagate" as="xs:boolean" tunnel="yes"/>
        <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else ."/>
    </xsl:template>
    
    <!-- ============ -->
    <!-- Phase 'flag' -->
    <!-- ============ -->
    
    <!-- Ermitteln, ob für den aktuellen Fehler eine Umschluesselungsanweisung (Hochgewichtung) vorliegt. Die 
        Umschluesselung steuert als Parameter die Verarbeitung von Fehlerschluessel, -gewicht und -Text ein.  
    -->
    
    <xsl:template match="fehler" mode="flag">
        <xsl:element name="fehler">
            <xsl:copy-of select="@*"/>
            <xsl:variable name="codeTransform" select="$applicableCodeTransforms[@alt eq current()/schluessel/text()]"/>
            <xsl:if test="$debug and not(empty($codeTransform))">
                <xsl:message select="('Transformiere Schluessel ',string($codeTransform/@alt), ' zu ', string($codeTransform/@neu))"/>
            </xsl:if>
            <xsl:apply-templates mode="flag">
                <xsl:with-param name="codeTransform" select="$codeTransform" tunnel="yes"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    
    <!-- Fehlerschluessel setzen -->
    
    <xsl:template match="fehler/schluessel/text()" mode="flag">
        <xsl:param name="codeTransform" as="element()?" tunnel="yes" required="no"/>
        <xsl:value-of select="if (empty($codeTransform)) then . else $codeTransform/@neu"/>
    </xsl:template>
    
    <!-- Fehlergewicht setzen -->
    
    <xsl:template match="fehler/gewicht/text()" mode="flag">
        <xsl:param name="codeTransform" as="element()?" tunnel="yes" required="no"/>
        <xsl:value-of select="if (empty($codeTransform)) then . else 'ERROR'"/>
    </xsl:template>
    
    <!-- Fehlertext setzen -->
    
    <xsl:template match="fehler/text/text()" mode="flag">
        <xsl:param name="codeTransform" as="element()?" tunnel="yes"/>
        <xsl:variable name="text" select="$codeTransform/@text"/>
        <xsl:value-of select="if (empty($text)) then . else func:resolveVariables($text,ancestor::meldung,$codeTransform)"/>
    </xsl:template>
    
    <!-- Index von 'satz' in XPath-ausdruck durch UUID ersetzen, falls vorhanden -->
    
    <xsl:template match="fehler/position/text()" mode="flag">
        
        <xsl:variable name="xpath-mit-uuid" as="item()?">
            <xsl:if test="$document-processible and contains(.,'satz[')">
                
                <!-- Index des Elementes nachricht lesen -->
                
                <xsl:variable name="index-nachricht"
                    select="floor(number(substring-before(substring-after(.,'nachricht['),']')))"
                />
                
                <!-- Index des Elementes segement lesen -->
                
                <xsl:variable name="index-segment" select="if (contains(.,'segment['))
                        then floor(number(substring-before(substring-after(.,'segment['),']')))
                        else 0"
                />
                
                <!-- Index des Elementes satz lesen -->
                
                <xsl:variable name="index-satz"
                    select="floor(number(substring-before(substring-after(.,'satz['),']')))"
                />
                
                <!-- UUID lesen -->
                
                <xsl:variable name="uuid"
                    select="if ($index-segment gt 0)
                    then $datml-raw-doc-intern/DatML-RAW-D/nachricht[$index-nachricht]/segment[$index-segment]/datensegment/satz[$index-satz]/mm[@name='UUID'][1]/wert/text()
                    else $datml-raw-doc-intern/DatML-RAW-D/nachricht[$index-nachricht]//datensegment/satz[$index-satz]/mm[@name='UUID'][1]/wert/text()"
                />
                <!--
                <xsl:message select="$index-segment"/>
                <xsl:message select="$uuid"/>
                -->
                
                <!-- XPath mit UUID konstruieren -->
                
                <xsl:if test="not(empty($uuid))">
                    <xsl:variable name="nachricht-mit-index" select="string-join(('nachricht[',string($index-nachricht),']'),'')"/>
                    <xsl:variable name="satz-mit-index" select="string-join(('satz[',string($index-satz),']'),'')"/>
                    <xsl:variable name="satz-mit-uuid" select="string-join(('satz[mm[@name=''UUID''][wert=''',string($uuid),''']]'),'')"/>
                    <xsl:variable name="pfad-bis-nachricht" select="substring-before(.,$nachricht-mit-index)"/>
                    <xsl:variable name="pfad-nachricht-satz" select="substring-after(substring-before(.,$satz-mit-index),$nachricht-mit-index)"/>
                    <xsl:variable name="pfad-satz-merkmal" select="substring-after(.,$satz-mit-index)"/>
                    <!--
                    <xsl:message select="(string-join(($nachricht-mit-index,'/',$satz-mit-index),''),':',$satz-mit-uuid),':',$pfad-satz-merkmal"/>
                    -->
                    <xsl:value-of select="string-join((
                        $pfad-bis-nachricht,
                        $nachricht-mit-index,
                        $pfad-nachricht-satz,$satz-mit-uuid,
                        $pfad-satz-merkmal
                        ),'')"
                    />
                </xsl:if>
                
            </xsl:if>
        </xsl:variable>
        
        <!-- (Modifizierten) Pfad ausgeben -->
        
        <xsl:value-of select="if (empty($xpath-mit-uuid)) then . else $xpath-mit-uuid"/>
        
    </xsl:template>
    
    <!-- Umkodierungsanweisung fuer angegebene Bezugsdatum des Berichtszeitraumes suchen -->
    
    <xsl:template name="matchCodeTransform" as="element()?">
        <xsl:param name="codeTransforms" as="element()*" required="yes"/>
        <xsl:param name="refDate" as="xs:date" required="yes"/>
        <xsl:choose>
            <xsl:when test="not(empty($codeTransforms))">
                <xsl:variable name="codeTransform" select="$codeTransforms[1]" as="element()"/>
                <xsl:choose>
                    <xsl:when test="not($codeTransform/@gueltigAb) or $refDate ge func:referencePeriod2Date($codeTransform/@gueltigAb)" >
                        <xsl:sequence select="$codeTransform"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="matchCodeTransform">
                            <xsl:with-param name="codeTransforms" select="subsequence($codeTransforms,2)"/>
                            <xsl:with-param name="refDate" select="$refDate"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:template>
    
    <!-- ============ -->
    <!-- Phase 'text' -->
    <!-- ============ -->
    
    <!-- Anwendung: Nname setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/anwendungsname/text()" mode="text">
        <xsl:variable name="name" select="($config-file-intern/res2res-config/anwendung/name/text())"/>
        <xsl:value-of select="if (empty($name)) then . else $name"/>
    </xsl:template>
    
    <!-- Anwendung: Version setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/version/text()" mode="text">
        <xsl:variable name="version" select="($config-file-intern/res2res-config/anwendung/version/text())"/>
        <xsl:value-of select="if (empty($version)) then . else $version"/>
    </xsl:template>
    
    <!-- Anwendung: Hersteller setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/hersteller/text()" mode="text">
        <xsl:variable name="hersteller" select="($config-file-intern/res2res-config/anwendung/hersteller/text())"/>
        <xsl:value-of select="if (empty($hersteller)) then . else $hersteller"/>
    </xsl:template>
    
    <!-- Datenformat: Auf 'xml' setzen -->
    
    <xsl:template match="dokument/dateneingang/datei/datenformat/@typ" mode="text">
        <xsl:attribute name="typ">xml</xsl:attribute>
    </xsl:template>
    
    <!-- Dokument: Hinweistext anpassen -->
    
    <xsl:template match="dokument/verarbeitungshinweis/text()" mode="text">
        <xsl:value-of select="func:getMessage('hinweis',$processingMode,../../@dokumentstatus)"/>
    </xsl:template>
    
    <!-- Dokument: Erlaeuterungstext anpassen -->
    
    <xsl:template match="dokument/erlaeuterung/text()" mode="text">
        <xsl:value-of select="func:getMessage('erlaeuterung',$processingMode,../../@dokumentstatus)"/>
    </xsl:template>
    
    <!-- Meldung: Heinweistext anpassen -->
    
    <xsl:template match="meldung/verarbeitungshinweis/text()" mode="text">
        <xsl:value-of select="func:getMessage('hinweis',$processingMode,../../@meldungsstatus)"/>
    </xsl:template>
    
    <!-- Meldung: Erlaeuterungstext anpassen -->
    
    <xsl:template match="meldung/erlaeuterung/text()" mode="text">
        <xsl:value-of select="func:getMessage('erlaeuterung',$processingMode,../../@meldungsstatus)"/>
    </xsl:template>
    
    <!-- ========== -->
    <!-- Funktionen -->
    <!-- ========== -->
    
    <!-- Meldungstext aus Konfigurationsdatei lesen -->
    
    <xsl:function name="func:getMessage" as="xs:string">
        <xsl:param name="type" as="xs:string" required="yes"/>
        <xsl:param name="mode" as="xs:string" required="yes"/>
        <xsl:param name="status" as="xs:string" required="yes"/>
        <xsl:value-of select="($config-file-intern/res2res-config/texte[@typ eq $type][@modus eq $mode]/text[@status eq $status]/text())"/>
    </xsl:function>
    
    <!-- Variablen ersetzen -->
    
    <xsl:function name="func:resolveVariables" as="xs:string?">
        <xsl:param name="text" as="xs:string?"/>
        <xsl:param name="meldung" as="element()?"/>
        <xsl:param name="codeTransform" as="element()?" required="no"/>
        <xsl:value-of>
            <xsl:analyze-string select="$text" regex="\$\{{[^}}]+\}}">
                <xsl:matching-substring>
                    <xsl:variable name="var" select="substring-before(substring-after(.,'${'),'}')" as="xs:string"/>
                    <xsl:choose>
                        <xsl:when test="$var eq 'erhebung'">
                            <xsl:value-of select="if (empty($meldung)) then func:unresolved(.) else $meldung/erhebung/kennung"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'erhebung.klasse'">
                            <xsl:value-of select="if (empty($meldung)) then func:unresolved(.) else $meldung/erhebung/kennung/@klasse"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'berichtszeitraum'">
                            <xsl:value-of select="if (empty($meldung)) then func:unresolved(.) else string-join($meldung/berichtszeitraum/*/text(),'-')"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'gueltig.ab'">
                            <xsl:value-of select="if (empty($codeTransform)) then func:unresolved(.) else $codeTransform/@gueltigAb"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'schluessel.alt'">
                            <xsl:value-of select="if (empty($codeTransform)) then func:unresolved(.) else $codeTransform/@alt"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'schluessel.neu'">
                            <xsl:value-of select="if (empty($codeTransform)) then func:unresolved(.) else $codeTransform/@neu"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="func:unmatched(.)"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:matching-substring>
                <xsl:non-matching-substring>
                    <xsl:value-of select="."/>
                </xsl:non-matching-substring>
            </xsl:analyze-string>
        </xsl:value-of>
    </xsl:function>
    
    <!-- Ersatzwert fuer nicht ersetzbare Variablenreferenz ausgeben -->
    
    <xsl:function name="func:unresolved" as="xs:string">
        <xsl:param name="varname" as="xs:string"/>
        <xsl:value-of select="concat('**KEIN WERT FUER:',$varname,'**')"/>
    </xsl:function>
    
    <!-- Ersatzwert fuer nicht bekannte Variablenreferenz ausgeben -->
    
    <xsl:function name="func:unmatched" as="xs:string">
        <xsl:param name="varname" as="xs:string"/>
        <xsl:value-of select="concat('**UNBEKANNTE VARIABLE:',$varname,'**')"/>
    </xsl:function>
    
    <!-- Zugriff auf Dokument pruefen -->
    
    <xsl:function name="func:checkDocument" as="xs:boolean">
        <xsl:param name="document" required="yes"/>
        <xsl:param name="hint" as="xs:string"/>
        <xsl:choose>
            <xsl:when test="doc-available($document)">
                <xsl:sequence select="true()"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message select="('*** Kein Zugriff auf das Dokument ''',$hint,''' an ''',$document,'''')"/>
                <xsl:sequence select="false()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <!-- Ermitteln der Umkodierungsanweisung fuer das Bezugsdatum des Berichtszeitraumes
        Aus der uebergebenen Gruppe von Umkodierungsanweisungen wird die erste zum angegebenen Datum
        passende mittels Rekursion (Template matchCodeTransform) ermittelt.
    -->
    
    <xsl:function name="func:findCodeTransformForPeriod" as="element()?">
        <xsl:param name="referencePeriod" as="xs:string" required="yes"/>
        <xsl:param name="codeTransforms" as="element()*" required="yes"/>
        <xsl:variable name="refDate" as="xs:date" select="func:referencePeriod2Date($referencePeriod)"/>
        <xsl:call-template name="matchCodeTransform">
            <xsl:with-param name="codeTransforms" select="func:sortCodeTransforms($codeTransforms)"/>
            <xsl:with-param name="refDate" select="$refDate"/>
        </xsl:call-template>
    </xsl:function>
    
    <!-- Alle Umkodierungsanweisungen fuer den angegebenen Bezugsdatum des Berichtszeitraumes suchen
        Fuer jeder Gruppe von Umkodierungsanweisungen mit dem selben Schluessel wird jeweils der
        fuer den Berichtszeitraum zutreffende ermittelt, falls vorhanden. Berichtszeitraeume werden
        hierzu in ein Datum (Monatserster) umgerechnet.
    -->
    
    <xsl:function name="func:findAllCodeTransformsForPeriod" as="element()*">
        <xsl:param name="referencePeriod" as="xs:string" required="yes"/>
        <xsl:param name="codeTransforms" as="element()*" required="yes"/>
        <xsl:variable name="refDate" as="xs:date" select="func:referencePeriod2Date($referencePeriod)"/>
        <xsl:for-each-group select="$codeTransforms" group-by="@alt">
            <xsl:sequence select="func:findCodeTransformForPeriod($referencePeriod,current-group())"/>
        </xsl:for-each-group>
    </xsl:function>
    
    <!-- Umkodierungsanweisungen absteigend nach Berichtszeitraum sortieren -->
    
    <xsl:function name="func:sortCodeTransforms" as="element()*">
        <xsl:param name="codeTransforms" as="element()*" required="yes"/>
        <xsl:perform-sort select="$codeTransforms">
            <xsl:sort select="@gueltigAb" order="descending"/>
        </xsl:perform-sort>
    </xsl:function>
    
    <!-- Berichtszeitraum zu String yyyy-mm konvertieren -->
    
    <xsl:function name="func:berichtszeitraum2String" as="xs:string">
        <xsl:param name="berichtszeitraum" as="element()" required="yes"/>
        <xsl:value-of select="concat(
            format-number($berichtszeitraum/jahr/text(),'0000'),'-',
            format-number($berichtszeitraum/monat/text(),'00')
            )
        "/>
    </xsl:function>
    
    <!-- Berichtszeitraum yyyy-mm in Monatsersten umrechnen -->
    
    <xsl:function name="func:referencePeriod2Date" as="xs:date">
        <xsl:param name="referencePeriod" as="xs:string" required="yes"/>
        <xsl:sequence select="xs:date(concat($referencePeriod,'-01'))"/>
    </xsl:function>
    
    <!-- 
        Identity transform 
        ### 'mode="#all"' UND 'mode="#current"' MUESSEN GESETZT SEIN ###
    -->
    
    <xsl:template match="node()|@*" mode="#all">
        <!-- Copy the context item -->
        <xsl:copy>
            <!-- Copy any attributes and any child nodes -->
            <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>